@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2007  NTT DATA CORPORATION
  *
- * Version: 1.4   2007/04/01
+ * Version: 1.5.1   2007/10/19
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -24,7 +24,6 @@ extern struct semaphore domain_acl_lock;
 
 /*************************  AUDIT FUNCTIONS  *************************/
 
-#ifdef CONFIG_TOMOYO_AUDIT
 static int AuditNetworkLog(const int is_ipv6, const char *operation, const u32 *address, const u16 port, const int is_granted)
 {
 	char *buf;
@@ -41,19 +40,16 @@ static int AuditNetworkLog(const int is_ipv6, const char *operation, const u32 *
 	snprintf(buf + strlen(buf), len - strlen(buf) - 1, " %u\n", port);
 	return WriteAuditLog(buf, is_granted);
 }
-#else
-static inline void AuditNetworkLog(const int is_ipv6, const char *operation, const u32 *address, const u16 port, const int is_granted) {}
-#endif
 
 /*************************  ADDRESS GROUP HANDLER  *************************/
 
-static ADDRESS_GROUP_ENTRY *group_list = NULL;
+static struct address_group_entry *group_list = NULL;
 
 static int AddAddressGroupEntry(const char *group_name, const u8 is_ipv6, const u16 *min_address, const u16 *max_address, const int is_delete)
 {
 	static DECLARE_MUTEX(lock);
-	ADDRESS_GROUP_ENTRY *new_group, *group;
-	ADDRESS_GROUP_MEMBER *new_member, *member;
+	struct address_group_entry *new_group, *group;
+	struct address_group_member *new_member, *member;
 	const struct path_info *saved_group_name;
 	int error = -ENOMEM;
 	if (!IsCorrectPath(group_name, 0, 0, 0, __FUNCTION__) || !group_name[0]) return -EINVAL;
@@ -79,7 +75,7 @@ static int AddAddressGroupEntry(const char *group_name, const u8 is_ipv6, const
 		goto out;
 	}
 	if (!group) {
-		if ((new_group = (ADDRESS_GROUP_ENTRY *) alloc_element(sizeof(ADDRESS_GROUP_ENTRY))) == NULL) goto out;
+		if ((new_group = alloc_element(sizeof(*new_group))) == NULL) goto out;
 		new_group->group_name = saved_group_name;
 		mb(); /* Instead of using spinlock. */
 		if ((group = group_list) != NULL) {
@@ -89,7 +85,7 @@ static int AddAddressGroupEntry(const char *group_name, const u8 is_ipv6, const
 		}
 		group = new_group;
 	}
-	if ((new_member = (ADDRESS_GROUP_MEMBER *) alloc_element(sizeof(ADDRESS_GROUP_MEMBER))) == NULL) goto out;
+	if ((new_member = alloc_element(sizeof(*new_member))) == NULL) goto out;
 	new_member->is_ipv6 = is_ipv6;
 	if (is_ipv6) {
 		memmove(new_member->min.ipv6, min_address, 16);
@@ -143,10 +139,10 @@ int AddAddressGroupPolicy(char *data, const int is_delete)
 	return AddAddressGroupEntry(data, is_ipv6, min_address, max_address, is_delete);
 }
 
-static ADDRESS_GROUP_ENTRY *FindOrAssignNewAddressGroup(const char *group_name)
+static struct address_group_entry *FindOrAssignNewAddressGroup(const char *group_name)
 {
 	int i;
-	ADDRESS_GROUP_ENTRY *group;
+	struct address_group_entry *group;
 	for (i = 0; i <= 1; i++) {
 		for (group = group_list; group; group = group->next) {
 			if (strcmp(group_name, group->group_name->name) == 0) return group;
@@ -160,9 +156,9 @@ static ADDRESS_GROUP_ENTRY *FindOrAssignNewAddressGroup(const char *group_name)
 	return NULL;
 }
 
-static int AddressMatchesToGroup(const u8 is_ipv6, const u32 *address, const ADDRESS_GROUP_ENTRY *group)
+static int AddressMatchesToGroup(const u8 is_ipv6, const u32 *address, const struct address_group_entry *group)
 {
-	ADDRESS_GROUP_MEMBER *member;
+	struct address_group_member *member;
 	const u32 ip = ntohl(*address);
 	for (member = group->first_member; member; member = member->next) {
 		if (member->is_deleted) continue;
@@ -175,16 +171,16 @@ static int AddressMatchesToGroup(const u8 is_ipv6, const u32 *address, const ADD
 	return 0;
 }
 
-int ReadAddressGroupPolicy(IO_BUFFER *head)
+int ReadAddressGroupPolicy(struct io_buffer *head)
 {
-	ADDRESS_GROUP_ENTRY *group = (ADDRESS_GROUP_ENTRY *) head->read_var1;
-	ADDRESS_GROUP_MEMBER *member = (ADDRESS_GROUP_MEMBER *) head->read_var2;
+	struct address_group_entry *group = head->read_var1;
+	struct address_group_member *member = head->read_var2;
 	if (!group) group = group_list;
 	while (group) {
-		head->read_var1 = (struct domain_info *) group;
+		head->read_var1 = group;
 		if (!member) member = group->first_member;
 		while (member) {
-			head->read_var2 = (void *) member;
+			head->read_var2 = member;
 			if (!member->is_deleted) {
 				char buf[128];
 				if (member->is_ipv6) {
@@ -256,34 +252,34 @@ const char *network2keyword(const unsigned int operation)
 	return keyword;
 }
 
-static int AddNetworkEntry(const u8 operation, const u8 record_type, const struct address_group_entry *group, const u32 *min_address, const u32 *max_address, const u16 min_port, const u16 max_port, struct domain_info *domain, const u8 is_add, const struct condition_list *condition)
+static int AddNetworkEntry(const u8 operation, const u8 record_type, const struct address_group_entry *group, const u32 *min_address, const u32 *max_address, const u16 min_port, const u16 max_port, struct domain_info *domain, const struct condition_list *condition, const u8 is_delete)
 {
 	struct acl_info *ptr;
 	int error = -ENOMEM;
 	const u32 min_ip = ntohl(*min_address), max_ip = ntohl(*max_address); /* using host byte order to allow u32 comparison than memcmp().*/
 	if (!domain) return -EINVAL;
 	down(&domain_acl_lock);
-	if (is_add) {
+	if (!is_delete) {
 		if ((ptr = domain->first_acl_ptr) == NULL) goto first_entry;
 		while (1) {
-			IP_NETWORK_ACL_RECORD *new_ptr;
-			if (ptr->type == TYPE_IP_NETWORK_ACL && ptr->u.b[0] == operation && ptr->u.b[1] == record_type && ptr->cond == condition && ((IP_NETWORK_ACL_RECORD *) ptr)->min_port == min_port && max_port == ((IP_NETWORK_ACL_RECORD *) ptr)->max_port) {
+			struct ip_network_acl_record *new_ptr = (struct ip_network_acl_record *) ptr;
+			if (ptr->type == TYPE_IP_NETWORK_ACL && new_ptr->operation_type == operation && new_ptr->record_type == record_type && ptr->cond == condition && new_ptr->min_port == min_port && max_port == new_ptr->max_port) {
 				if (record_type == IP_RECORD_TYPE_ADDRESS_GROUP) {
-					if (((IP_NETWORK_ACL_RECORD *) ptr)->u.group == group) {
+					if (new_ptr->u.group == group) {
 						ptr->is_deleted = 0;
 						/* Found. Nothing to do. */
 						error = 0;
 						break;
 					}
 				} else if (record_type == IP_RECORD_TYPE_IPv4) {
-					if (((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv4.min == min_ip && max_ip == ((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv4.max) {
+					if (new_ptr->u.ipv4.min == min_ip && max_ip == new_ptr->u.ipv4.max) {
 						ptr->is_deleted = 0;
 						/* Found. Nothing to do. */
 						error = 0;
 						break;
 					}
 				} else if (record_type == IP_RECORD_TYPE_IPv6) {
-					if (memcmp(((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv6.min, min_address, 16) == 0 && memcmp(max_address, ((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv6.max, 16) == 0) {
+					if (memcmp(new_ptr->u.ipv6.min, min_address, 16) == 0 && memcmp(max_address, new_ptr->u.ipv6.max, 16) == 0) {
 						ptr->is_deleted = 0;
 						/* Found. Nothing to do. */
 						error = 0;
@@ -296,12 +292,11 @@ static int AddNetworkEntry(const u8 operation, const u8 record_type, const struc
 				continue;
 			}
 		first_entry: ;
-			if (is_add == 1 && TooManyDomainACL(domain)) break;
 			/* Not found. Append it to the tail. */
-			if ((new_ptr = (IP_NETWORK_ACL_RECORD *) alloc_element(sizeof(IP_NETWORK_ACL_RECORD))) == NULL) break;
+			if ((new_ptr = alloc_element(sizeof(*new_ptr))) == NULL) break;
 			new_ptr->head.type = TYPE_IP_NETWORK_ACL;
-			new_ptr->head.u.b[0] = operation;
-			new_ptr->head.u.b[1] = record_type;
+			new_ptr->operation_type = operation;
+			new_ptr->record_type = record_type;
 			new_ptr->head.cond = condition;
 			if (record_type == IP_RECORD_TYPE_ADDRESS_GROUP) {
 				new_ptr->u.group = group;
@@ -320,13 +315,14 @@ static int AddNetworkEntry(const u8 operation, const u8 record_type, const struc
 	} else {
 		error = -ENOENT;
 		for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
-			if (ptr->type != TYPE_IP_NETWORK_ACL || ptr->is_deleted || ptr->u.b[0] != operation || ptr->u.b[1] != record_type || ptr->cond != condition || ((IP_NETWORK_ACL_RECORD *) ptr)->min_port != min_port || ((IP_NETWORK_ACL_RECORD *) ptr)->max_port != max_port) continue;
+			struct ip_network_acl_record *ptr2 = (struct ip_network_acl_record *) ptr;
+			if (ptr->type != TYPE_IP_NETWORK_ACL || ptr->is_deleted || ptr2->operation_type != operation || ptr2->record_type != record_type || ptr->cond != condition || ptr2->min_port != min_port || ptr2->max_port != max_port) continue;
 			if (record_type == IP_RECORD_TYPE_ADDRESS_GROUP) {
-				if (((IP_NETWORK_ACL_RECORD *) ptr)->u.group != group) continue;
+				if (ptr2->u.group != group) continue;
 			} else if (record_type == IP_RECORD_TYPE_IPv4) {
-				if (((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv4.min != min_ip || max_ip != ((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv4.max) continue;
+				if (ptr2->u.ipv4.min != min_ip || max_ip != ptr2->u.ipv4.max) continue;
 			} else if (record_type == IP_RECORD_TYPE_IPv6) {
-				if (memcmp(((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv6.min, min_address, 16) || memcmp(max_address, ((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv6.max, 16)) continue;
+				if (memcmp(ptr2->u.ipv6.min, min_address, 16) || memcmp(max_address, ptr2->u.ipv6.max, 16)) continue;
 			}
 			error = DelDomainACL(ptr);
 			break;
@@ -345,13 +341,14 @@ static int CheckNetworkEntry(const int is_ipv6, const int operation, const u32 *
 	const u32 ip = ntohl(*address); /* using host byte order to allow u32 comparison than memcmp().*/
 	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_NETWORK)) return 0;
 	for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
-		if (ptr->type != TYPE_IP_NETWORK_ACL || ptr->is_deleted || ptr->u.b[0] != operation || port < ((IP_NETWORK_ACL_RECORD *) ptr)->min_port || ((IP_NETWORK_ACL_RECORD *) ptr)->max_port < port || CheckCondition(ptr->cond, NULL)) continue;
-		if (ptr->u.b[1] == IP_RECORD_TYPE_ADDRESS_GROUP) {
-			if (AddressMatchesToGroup(is_ipv6, address, ((IP_NETWORK_ACL_RECORD *) ptr)->u.group)) break;
-		} else if (ptr->u.b[1] == IP_RECORD_TYPE_IPv4) {
-			if (!is_ipv6 && ((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv4.min <= ip && ip <= ((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv4.max) break;
+		struct ip_network_acl_record *ptr2 = (struct ip_network_acl_record *) ptr;
+		if (ptr->type != TYPE_IP_NETWORK_ACL || ptr->is_deleted || ptr2->operation_type != operation || port < ptr2->min_port || ptr2->max_port < port || CheckCondition(ptr->cond, NULL)) continue;
+		if (ptr2->record_type == IP_RECORD_TYPE_ADDRESS_GROUP) {
+			if (AddressMatchesToGroup(is_ipv6, address, ptr2->u.group)) break;
+		} else if (ptr2->record_type == IP_RECORD_TYPE_IPv4) {
+			if (!is_ipv6 && ptr2->u.ipv4.min <= ip && ip <= ptr2->u.ipv4.max) break;
 		} else {
-			if (is_ipv6 && memcmp(((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv6.min, address, 16) <= 0 && memcmp(address, ((IP_NETWORK_ACL_RECORD *) ptr)->u.ipv6.max, 16) <= 0) break;
+			if (is_ipv6 && memcmp(ptr2->u.ipv6.min, address, 16) <= 0 && memcmp(address, ptr2->u.ipv6.max, 16) <= 0) break;
 		}
 	}
 	if (ptr) {
@@ -376,7 +373,7 @@ static int CheckNetworkEntry(const int is_ipv6, const int operation, const u32 *
 		}
 		return CheckSupervisor("%s\n" KEYWORD_ALLOW_NETWORK "%s %u.%u.%u.%u %u\n", domain->domainname->name, keyword, HIPQUAD(ip), port);
 	}
-	if (CheckCCSAccept(CCS_TOMOYO_MAC_FOR_NETWORK)) AddNetworkEntry(operation, is_ipv6 ? IP_RECORD_TYPE_IPv6: IP_RECORD_TYPE_IPv4, NULL, address, address, port, port, domain, 1, NULL);
+	if (CheckCCSAccept(CCS_TOMOYO_MAC_FOR_NETWORK, domain)) AddNetworkEntry(operation, is_ipv6 ? IP_RECORD_TYPE_IPv6 : IP_RECORD_TYPE_IPv4, NULL, address, address, port, port, domain, NULL, 0);
 	return 0;
 }
 
@@ -438,7 +435,7 @@ int AddNetworkPolicy(char *data, struct domain_info *domain, const int is_delete
 	if (strchr(cp1, ' ')) goto out;
 	if ((count = sscanf(cp1, "%hu-%hu", &min_port, &max_port)) == 1 || count == 2) {
 		if (count == 1) max_port = min_port;
-		return AddNetworkEntry(operation, record_type, group, (u32 *) min_address, (u32 *) max_address, min_port, max_port, domain, is_delete ? 0 : -1, condition);
+		return AddNetworkEntry(operation, record_type, group, (u32 *) min_address, (u32 *) max_address, min_port, max_port, domain, condition, is_delete);
 	}
  out: ;
 	return -EINVAL;
@@ -448,37 +445,36 @@ int CheckNetworkListenACL(const int is_ipv6, const u8 *address, const u16 port)
 {
 	return CheckNetworkEntry(is_ipv6, NETWORK_ACL_TCP_LISTEN, (const u32 *) address, ntohs(port));
 }
+EXPORT_SYMBOL(CheckNetworkListenACL);
 
 int CheckNetworkConnectACL(const int is_ipv6, const int sock_type, const u8 *address, const u16 port)
 {
 	return CheckNetworkEntry(is_ipv6, sock_type == SOCK_STREAM ? NETWORK_ACL_TCP_CONNECT : (sock_type == SOCK_DGRAM ? NETWORK_ACL_UDP_CONNECT : NETWORK_ACL_RAW_CONNECT), (const u32 *) address, ntohs(port));
 }
+EXPORT_SYMBOL(CheckNetworkConnectACL);
 
 int CheckNetworkBindACL(const int is_ipv6, const int sock_type, const u8 *address, const u16 port)
 {
 	return CheckNetworkEntry(is_ipv6, sock_type == SOCK_STREAM ? NETWORK_ACL_TCP_BIND : (sock_type == SOCK_DGRAM ? NETWORK_ACL_UDP_BIND : NETWORK_ACL_RAW_BIND), (const u32 *) address, ntohs(port));
 }
+EXPORT_SYMBOL(CheckNetworkBindACL);
 
 int CheckNetworkAcceptACL(const int is_ipv6, const u8 *address, const u16 port)
 {
 	return CheckNetworkEntry(is_ipv6, NETWORK_ACL_TCP_ACCEPT, (const u32 *) address, ntohs(port));
 }
+EXPORT_SYMBOL(CheckNetworkAcceptACL);
 
 int CheckNetworkSendMsgACL(const int is_ipv6, const int sock_type, const u8 *address, const u16 port)
 {
 	return CheckNetworkEntry(is_ipv6, sock_type == SOCK_DGRAM ? NETWORK_ACL_UDP_CONNECT : NETWORK_ACL_RAW_CONNECT, (const u32 *) address, ntohs(port));
 }
+EXPORT_SYMBOL(CheckNetworkSendMsgACL);
 
 int CheckNetworkRecvMsgACL(const int is_ipv6, const int sock_type, const u8 *address, const u16 port)
 {
 	return CheckNetworkEntry(is_ipv6, sock_type == SOCK_DGRAM ? NETWORK_ACL_UDP_CONNECT : NETWORK_ACL_RAW_CONNECT, (const u32 *) address, ntohs(port));
 }
-
-EXPORT_SYMBOL(CheckNetworkListenACL);
-EXPORT_SYMBOL(CheckNetworkConnectACL);
-EXPORT_SYMBOL(CheckNetworkBindACL);
-EXPORT_SYMBOL(CheckNetworkAcceptACL);
-EXPORT_SYMBOL(CheckNetworkSendMsgACL);
 EXPORT_SYMBOL(CheckNetworkRecvMsgACL);
 
 /***** TOMOYO Linux end. *****/
